<?php

class ffStyleEnqueuer extends ffBasicObject {
/******************************************************************************/
/* VARIABLES AND CONSTANTS
/******************************************************************************/
	/**
	 * 
	 * @var ffWPLayer
	 */
	private $_WPLayer = null;
	
	/**
	 * 
	 * @var ffStyle_Factory
	 */
	private $_styleFactory = null;


	/**
	 * 
	 * @var array[ffStyle]
	 */
	protected $_styles = array();
	
	protected $_stylesNonMinificable = array();
	

	
/******************************************************************************/
/* CONSTRUCT AND PUBLIC FUNCTIONS
/******************************************************************************/
	public function __construct( ffWPLayer $WPLayer, ffStyle_Factory $styleFactory ) {
		$this->_setWplayer($WPLayer);
		$this->_setStylefactory($styleFactory);
		$this->_getWplayer()->getHookManager()->addActionEnqueuScripts(array( $this, 'actionEnqueueStyles' ));
	}
	
	public function addStyleFramework( $handle = null, $source = null, $dependencies = null, $version = null, $media = null, $type = null, $canBeMinified = true ) {
		$source = $this->_getWplayer()->getFrameworkUrl() . $source;
		$this->addStyle($handle, $source, $dependencies, $version, $media, $type, $canBeMinified );
	}
	
	public function addStyle( $handle = null, $source = null, $dependencies = null, $version = null, $media = null, $type = null, $canBeMinified = true ) {
		$style = $this->_getStylefactory()
						->createStyle( $handle, $source, $dependencies, $version, $media, $type, $canBeMinified);
		
		$this->_addStyle( $style );
		
	}
	
	public function addStyleObject( ffStyle $style ) {
		$this->_addStyle($style);
	}
	
	public function actionEnqueueStyles() {
		$this->_enqueueNonMinificableStyles();
		if( !empty($this->_styles) ) {
			foreach( $this->_styles as $oneStyle ) {
				$this->_getWplayer()
					->wp_enqueue_style(
							$oneStyle->handle,
							$oneStyle->source, 
							$oneStyle->dependencies, 
							null
					);
			}
		}
	}
/******************************************************************************/
/* PRIVATE FUNCTIONS
/******************************************************************************/
	protected function _enqueueNonMinificableStyles() {
		if( !empty($this->_stylesNonMinificable) ) {
			foreach( $this->_stylesNonMinificable as $oneStyle ) {
				$this->_getWplayer()
				->wp_enqueue_style(
						$oneStyle->handle,
						$oneStyle->source,
						$oneStyle->dependencies,
						null,
						$oneStyle->inFooter
				);
			}
		}
	}
	
	private function _addStyle( ffStyle $style ) {
		if( $style->canBeMinified == true ) {
			$this->_styles[ $style->handle ] = $style;
		} else {
			$this->_stylesNonMinificable[ $style->handle ] = $style;
		}
		
	}
/******************************************************************************/
/* SETTERS AND GETTERS
/******************************************************************************/	
	
	/**
	 * @return ffWPLayer
	 */
	protected function _getWplayer() {
		return $this->_WPLayer;
	}
	
	/**
	 * @param ffWPLayer $_WPLayer
	 */
	protected function _setWplayer(ffWPLayer $WPLayer) {
		$this->_WPLayer = $WPLayer;
		return $this;
	}
	
	/**
	 * @return ffStyle_Factory
	 */
	protected function _getStylefactory() {
		return $this->_styleFactory;
	}
	
	/**
	 * @param ffStyle_Factory $_styleFactory
	 */
	protected function _setStylefactory(ffStyle_Factory $styleFactory) {
		$this->_styleFactory = $styleFactory;
		return $this;
	}		
}