<?php

/**
 * Automatically load wished classes. Replacing SPL_AUTOLOAD which we should
 * not use due to compatibility issues.
 * 
 * @author FRESHFACE
 * @since 0.1
 *
 */

class ffClassLoader {
/******************************************************************************/
/* VARIABLES AND CONSTANTS
/******************************************************************************/
	/**
	 * array[ className ] = classPath
	 */
	private $_classNameToPathMap = array();
	
	/**
	 * 
	 * array[] = loadedClass
	 */
	private $_loadedClasses = array();
/******************************************************************************/
/* CONSTRUCT AND PUBLIC FUNCTIONS
/******************************************************************************/
	public function __construct() {
		$this->_initFrameworkClasses();
	}
	
	public function classRegistered( $className ) {
		return isset( $this->_classNameToPathMap[ $className ] );
	}
	
	public function classExists( $className ) {
		return class_exists( $className );
	}
	
	public function loadClass( $name ) {
		if( class_exists( $name ) ) {
			return true;
		}
		if( !in_array( $name, $this->_loadedClasses ) ) {
			if( !isset( $this->_classNameToPathMap[ $name ] ) ) {
				throw new Exception('ffClassLoader->loadClass trying to load class "'.$name.'" which has not been registered yet');
			}
			
			$path = $this->_classNameToPathMap[ $name ];
			$this->_requireClass( $path );
			$this->_loadedClasses[] = $name;
		}
	}
	
	public function addClass( $name, $path ) {
		$this->_classNameToPathMap[ $name ] = $path;
	}
/******************************************************************************/
/* PRIVATE FUNCTIONS
/******************************************************************************/
	############################################################################
	# CORE
	############################################################################
	private function _addCoreFiles() {
		$this->addClass( 'ffBasicObject', FF_FRAMEWORK_DIR.'/framework/core/class.ffBasicObject.php');
		$this->addClass( 'ffContainer', FF_FRAMEWORK_DIR.'/framework/core/class.ffContainer.php');
		$this->addClass( 'ffFramework', FF_FRAMEWORK_DIR.'/framework/core/class.ffFramework.php');
		$this->addClass( 'ffWPLayer', FF_FRAMEWORK_DIR.'/framework/core/class.ffWPLayer.php');
		$this->addClass( 'ffPluginLoader_Factory', FF_FRAMEWORK_DIR.'/framework/core/factories/class.ffPluginLoader_Factory.php');
		$this->addClass( 'ffPluginLoader', FF_FRAMEWORK_DIR.'/framework/core/class.ffPluginLoader.php');
		$this->addClass( 'ffPluginIdentificator', FF_FRAMEWORK_DIR.'/framework/core/class.ffPluginIdentificator.php');
		$this->addClass( 'ffFactoryAbstract', FF_FRAMEWORK_DIR.'/framework/core/class.ffFactoryAbstract.php');
		$this->addClass( 'ffPluginAbstract', FF_FRAMEWORK_DIR.'/framework/core/class.ffPluginAbstract.php');
		$this->addClass( 'ffPluginContainerAbstract', FF_FRAMEWORK_DIR.'/framework/core/class.ffPluginContainerAbstract.php');
		$this->addClass( 'ffRequest', FF_FRAMEWORK_DIR.'/framework/core/class.ffRequest.php');
		$this->addClass( 'ffRequest_Factory', FF_FRAMEWORK_DIR.'/framework/core/factories/class.ffRequest_Factory.php');
		$this->addClass( 'ffLibManager', FF_FRAMEWORK_DIR.'/framework/core/class.ffLibManager.php');
		$this->addClass( 'ffHookManager', FF_FRAMEWORK_DIR.'/framework/core/class.ffHookManager.php');
	}
	
	############################################################################
	# ASSETS
	############################################################################
	private function _addAssetsFiles() {
		$this->addClass( 'ffScript', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffScript.php');
		$this->addClass( 'ffScript_Factory', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/factories/class.ffScript_Factory.php');
		
		$this->addClass( 'ffScriptEnqueuer', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffScriptEnqueuer.php');
		$this->addClass( 'ffScriptEnqueuerMinification', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffScriptEnqueuerMinification.php');
		$this->addClass( 'ffScriptEnqueuer_Factory', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/factories/class.ffScriptEnqueuer_Factory.php');
		
		
		$this->addClass( 'ffStyle', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffStyle.php');
		$this->addClass( 'ffStyle_Factory', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/factories/class.ffStyle_Factory.php');
		$this->addClass( 'ffStyleEnqueuer', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffStyleEnqueuer.php');
		$this->addClass( 'ffStyleEnqueuerMinification', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffStyleEnqueuerMinification.php');
		$this->addClass( 'ffMinificator', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffMinificator.php');

		$this->addClass( 'ffMimeTypesManager', FF_FRAMEWORK_DIR.'/framework/assetsIncluding/class.ffMimeTypesManager.php');
	}
	
	############################################################################
	# FILE SYSTEM
	############################################################################	
	private function _addFileSystemFiles() {
		$this->addClass( 'ffFileManager_Factory', FF_FRAMEWORK_DIR.'/framework/fileSystem/factories/class.ffFileManager_Factory.php');
		$this->addClass( 'ffFileManager', FF_FRAMEWORK_DIR.'/framework/fileSystem/class.ffFileManager.php');
		$this->addClass( 'ffFileSystem', FF_FRAMEWORK_DIR.'/framework/fileSystem/class.ffFileSystem.php');
		$this->addClass( 'ffFileSystem_Factory', FF_FRAMEWORK_DIR.'/framework/fileSystem/factories/class.ffFileSystem_Factory.php');
		$this->addClass( 'ffHttp', FF_FRAMEWORK_DIR.'/framework/fileSystem/class.ffHttp.php');
		$this->addClass( 'ffFtp', FF_FRAMEWORK_DIR.'/framework/fileSystem/class.ffFtp.php');
		$this->addClass( 'ffPluginInstaller', FF_FRAMEWORK_DIR.'/framework/fileSystem/class.ffPluginInstaller.php');
		$this->addClass( 'ffHtaccess', FF_FRAMEWORK_DIR.'/framework/fileSystem/class.ffHtaccess.php');
	}
	
	############################################################################
	# OPTIONS
	############################################################################	
	private function _addOptionsFiles() {
		########################################################################
		# dataHolders
		########################################################################
		$this->addClass( 'ffIOneDataNode', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffIOneDataNode.php');
		$this->addClass( 'ffOneOption', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffOneOption.php');
		$this->addClass( 'ffOneElement', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffOneElement.php');
		$this->addClass( 'ffOneSection', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffOneSection.php');
		$this->addClass( 'ffOneStructure', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffOneStructure.php');
		$this->addClass( 'ffIOptionsHolder', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffIOptionsHolder.php');
		$this->addClass( 'ffOptionsHolder', FF_FRAMEWORK_DIR.'/framework/options/dataHolders/class.ffOptionsHolder.php');
		
		########################################################################
		# factories
		########################################################################
		$this->addClass( 'ffOneOption_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOneOption_Factory.php');
		$this->addClass( 'ffOneSection_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOneSection_Factory.php');
		$this->addClass( 'ffOneStructure_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOneStructure_Factory.php');
		$this->addClass( 'ffOptions_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOptions_Factory.php');
		$this->addClass( 'ffOptionsQuery_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOptionsQuery_Factory.php');
		$this->addClass( 'ffOptionsArrayConvertor_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOptionsArrayConvertor_Factory.php');
		$this->addClass( 'ffOptionsHolder_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOptionsHolder_Factory.php');
		$this->addClass( 'ffOptionsPrinterComponent_Factory', FF_FRAMEWORK_DIR.'/framework/options/factories/class.ffOptionsPrinterComponent_Factory.php');
		
		
		########################################################################
		# printerComponent
		########################################################################
		
		$this->addClass( 'ffOptionsPrinterComponents', FF_FRAMEWORK_DIR.'/framework/options/printerComponent/class.ffOptionsPrinterComponents.php');
		$this->addClass( 'ffOptionsPrinterElements', FF_FRAMEWORK_DIR.'/framework/options/printerComponent/class.ffOptionsPrinterElements.php');
		
		
		########################################################################
		# walkers
		########################################################################
		$this->addClass( 'ffOptionsWalker', FF_FRAMEWORK_DIR.'/framework/options/walkers/class.ffOptionsWalker.php');
		$this->addClass( 'ffOptionsQuery', FF_FRAMEWORK_DIR.'/framework/options/walkers/class.ffOptionsQuery.php');
		$this->addClass( 'ffOptionsArrayConvertor', FF_FRAMEWORK_DIR.'/framework/options/walkers/class.ffOptionsArrayConvertor.php');
		$this->addClass( 'ffOptionsPrinter', FF_FRAMEWORK_DIR.'/framework/options/walkers/class.ffOptionsPrinter.php');
		
		
	}
	
	############################################################################
	# DATA STORAGE
	############################################################################
	private function _addDataStorageFiles() {
		$this->addClass( 'ffDataStorage_Factory', FF_FRAMEWORK_DIR.'/framework/dataStorage/factories/class.ffDataStorage_Factory.php');
		$this->addClass( 'ffIDataStorage', FF_FRAMEWORK_DIR.'/framework/dataStorage/class.ffIDataStorage.php');
		$this->addClass( 'ffDataStorage', FF_FRAMEWORK_DIR.'/framework/dataStorage/class.ffDataStorage.php');
		$this->addClass( 'ffDataStorage_WPOptions', FF_FRAMEWORK_DIR.'/framework/dataStorage/class.ffDataStorage_WPOptions.php');
		$this->addClass( 'ffDataStorage_WPOptions_NamespaceFacade', FF_FRAMEWORK_DIR.'/framework/dataStorage/class.ffDataStorage_WPOptions_NamespaceFacade.php');
		
		$this->addClass( 'ffDataStorage_Cache', FF_FRAMEWORK_DIR.'/framework/dataStorage/class.ffDataStorage_Cache.php');
	}
	
	############################################################################
	# LIBS
	############################################################################
	private function _addLibsFiles() {
		########################################################################
		# TWITTER
		########################################################################
		$this->addClass('ffOptionsHolder_Twitter', FF_FRAMEWORK_DIR.'/framework/lib/twitter/class.ffOptionsHolder_Twitter.php');
		$this->addClass('ffLib_TwitterFeeder', FF_FRAMEWORK_DIR.'/framework/lib/twitter/class.ffLib_TwitterFeeder.php');
		$this->addClass('ffLib_TwitterFeeder_OAuthFactory', FF_FRAMEWORK_DIR.'/framework/lib/twitter/factories/class.ffLib_TwitterFeeder_OAuthFactory.php');
		$this->addClass('extTwitterOAuth', FF_FRAMEWORK_DIR.'/framework/lib/twitter/class.extTwitterOAuth.php');
		$this->addClass('ffLib_TwitterFeeder_OneTweet', FF_FRAMEWORK_DIR.'/framework/lib/twitter/class.ffLib_TwitterFeeder_OneTweet.php');
		$this->addClass('ffLib_TwitterFeeder_OneTweet_Factory', FF_FRAMEWORK_DIR.'/framework/lib/twitter/factories/class.ffLib_TwitterFeeder_OneTweet_Factory.php');
		$this->addClass('ffLib_TwitterFeeder_TweetsCollection', FF_FRAMEWORK_DIR.'/framework/lib/twitter/class.ffLib_TwitterFeeder_TweetsCollection.php');
		$this->addClass('ffLib_TwitterFeeder_TweetsCollection_Factory', FF_FRAMEWORK_DIR.'/framework/lib/twitter/factories/class.ffLib_TwitterFeeder_TweetsCollection_Factory.php');
	}

	############################################################################
	# COMPONENT
	############################################################################	
	private function _addComponentFiles() {
		$this->addClass( 'ffComponentAbstract', FF_FRAMEWORK_DIR.'/framework/components/class.ffComponentAbstract.php');
		$this->addClass( 'ffWidgetDecoratorAbstract', FF_FRAMEWORK_DIR.'/framework/components/class.ffWidgetDecoratorAbstract.php');
		
		
		$this->addClass( 'ffComponent_Factory', FF_FRAMEWORK_DIR.'/framework/components/factories/class.ffComponent_Factory.php');
		
		$this->addClass( 'ffWidgetManager', FF_FRAMEWORK_DIR.'/framework/components/class.ffWidgetManager.php');
	}
	
	############################################################################
	# ADMIN SCREEN FILE
	############################################################################
	private function _addAdminScreenFiles() {
		$this->addClass( 'ffMenuFactory', FF_FRAMEWORK_DIR.'/framework/adminScreens/factories/class.ffMenuFactory.php');
		$this->addClass( 'ffMenu', FF_FRAMEWORK_DIR.'/framework/adminScreens/class.ffMenu.php');
		$this->addClass( 'ffMenuManager', FF_FRAMEWORK_DIR.'/framework/adminScreens/class.ffMenuManager.php');
		$this->addClass( 'ffAdminScreenManager', FF_FRAMEWORK_DIR.'/framework/adminScreens/class.ffAdminScreenManager.php');
		$this->addClass( 'ffAdminScreen', FF_FRAMEWORK_DIR.'/framework/adminScreens/class.ffAdminScreen.php');
		$this->addClass( 'ffAdminScreenView', FF_FRAMEWORK_DIR.'/framework/adminScreens/class.ffAdminScreenView.php');
		$this->addClass( 'ffIAdminScreen', FF_FRAMEWORK_DIR.'/framework/adminScreens/interfaces/class.ffIAdminScreen.php');
		$this->addClass( 'ffIAdminScreenView', FF_FRAMEWORK_DIR.'/framework/adminScreens/interfaces/class.ffIAdminScreenView.php');
		$this->addClass( 'ffAdminScreenFactory', FF_FRAMEWORK_DIR.'/framework/adminScreens/factories/class.ffAdminScreenFactory.php');
		$this->addClass( 'ffAdminScreenViewFactory', FF_FRAMEWORK_DIR.'/framework/adminScreens/factories/class.ffAdminScreenViewFactory.php');
		
		$this->addClass( 'ffAdminScreenAjax', FF_FRAMEWORK_DIR.'/framework/adminScreens/class.ffAdminScreenAjax.php');
		$this->addClass( 'ffAdminScreenAjaxFactory', FF_FRAMEWORK_DIR.'/framework/adminScreens/factories/class.ffAdminScreenAjaxFactory.php');
		
	}

	############################################################################
	# EXTERN FILES
	############################################################################	
	private function _addExternFiles() {
		########################################################################
		# MINIFICATION
		########################################################################
		$this->addClass( 'externCssMin', FF_FRAMEWORK_DIR.'/framework/extern/minify/CSSmin.php');
		$this->addClass( 'externJsMinPlus', FF_FRAMEWORK_DIR.'/framework/extern/minify/JSMinPlus.php');
		$this->addClass( 'externJsMinPlus_Adapteur', FF_FRAMEWORK_DIR.'/framework/extern/minify/JSMinPlus_Adapteur.php');
		
		########################################################################
		# FTP
		########################################################################
		$this->addClass( 'externDgFtp', FF_FRAMEWORK_DIR.'/framework/extern/ftp/dgFtp.php');
		
		########################################################################
		# less
		########################################################################
		$this->addClass( 'lessCss', FF_FRAMEWORK_DIR.'/framework/extern/less/lessc.inc.php');
	}
	
	############################################################################
	# QUERY FILES
	############################################################################
	private function _addQueryFiles() {
		########################################################################
		# IDENTIFICATORS
		########################################################################
		$this->addClass( 'ffCustomPostTypeIdentificator', FF_FRAMEWORK_DIR.'/framework/query/identificators/post/class.ffCustomPostTypeIdentificator.php');
		$this->addClass( 'ffCustomPostTypeCollection', FF_FRAMEWORK_DIR.'/framework/query/identificators/post/class.ffCustomPostTypeCollection.php');
		$this->addClass( 'ffCustomPostTypeCollectionItem_Factory', FF_FRAMEWORK_DIR.'/framework/query/identificators/post/factories/class.ffCustomPostTypeCollectionItem_Factory.php');
		$this->addClass( 'ffCustomPostTypeCollection_Factory', FF_FRAMEWORK_DIR.'/framework/query/identificators/post/factories/class.ffCustomPostTypeCollection_Factory.php');
		$this->addClass( 'ffCustomPostTypeCollectionItem', FF_FRAMEWORK_DIR.'/framework/query/identificators/post/class.ffCustomPostTypeCollectionItem.php');
		
		$this->addClass( 'ffCustomTaxonomyIdentificator', FF_FRAMEWORK_DIR.'/framework/query/identificators/taxonomy/class.ffCustomTaxonomyIdentificator.php');
		$this->addClass( 'ffCustomTaxonomyCollection', FF_FRAMEWORK_DIR.'/framework/query/identificators/taxonomy/class.ffCustomTaxonomyCollection.php');
		$this->addClass( 'ffCustomTaxonomyCollectionItem', FF_FRAMEWORK_DIR.'/framework/query/identificators/taxonomy/class.ffCustomTaxonomyCollectionItem.php');
		
		$this->addClass( 'ffCustomTaxonomyCollection_Factory', FF_FRAMEWORK_DIR.'/framework/query/identificators/taxonomy/factories/class.ffCustomTaxonomyCollection_Factory.php');
		$this->addClass( 'ffCustomTaxonomyCollectionItem_Factory', FF_FRAMEWORK_DIR.'/framework/query/identificators/taxonomy/factories/class.ffCustomTaxonomyCollectionItem_Factory.php');
		
		########################################################################
		# GETTERS
		########################################################################
		$this->addClass( 'ffPostGetter', FF_FRAMEWORK_DIR.'/framework/query/getters/class.ffPostGetter.php');
		$this->addClass( 'ffTaxonomyGetter', FF_FRAMEWORK_DIR.'/framework/query/getters/class.ffTaxonomyGetter.php');
	}
	
	############################################################################ 
	# UPDATER FILES
	############################################################################
	private function _addUpdaterFiles() {
		$this->addClass('ffWPUpgrader', FF_FRAMEWORK_DIR.'/framework/updater/class.ffWPUpgrader.php');
	}
	
	private function _initFrameworkClasses() {
		$this->_addCoreFiles();
		$this->_addAssetsFiles();
		$this->_addFileSystemFiles();
		$this->_addOptionsFiles();
		$this->_addDataStorageFiles();
		$this->_addLibsFiles();
		$this->_addComponentFiles();
		$this->_addAdminScreenFiles();
		$this->_addExternFiles();
		$this->_addQueryFiles();
		$this->_addUpdaterFiles();
	}	
	
	private function _requireClass( $path ) {
		require_once( $path );
	}
/******************************************************************************/
/* SETTERS AND GETTERS
/******************************************************************************/
}