<?php
/**
 * Walking through the options and then printing them. Possible to set prefix
 * for printing ID and NAME of the options -> important for widgets mainly.
 * 
 * @author FRESHFACE
 * @since 0.1
 */
class ffOptionsPrinter extends ffOptionsWalker {
/******************************************************************************/
/* VARIABLES AND CONSTANTS
/******************************************************************************/
	/**
	 * 
	 * @var ffOptionsPrinterComponent_Factory
	 */
	private $_printerComponentFactory = null;
	
	/**
	 * 
	 * @var Prefix for name -> name="PREFIX[x][y]"
	 */
	private $_namePrefix = null;
	
	/**
	 * 
	 * @var prefix for ID -> id="PREFIXaaaaaa"
	 */
	private $_idPrefix = null;
	
/******************************************************************************/
/* CONSTRUCT AND PUBLIC FUNCTIONS
/******************************************************************************/
	/**
	 * Printer factory is important, since we want to print options
	 * 
	 * @param string $optionsArrayData
	 * @param string $optionsStructure
	 * @param ffOptionsPrinterComponent_Factory $printerComponentFactory
	 */
	public function __construct( $optionsArrayData = null, $optionsStructure = null, ffOptionsPrinterComponent_Factory $printerComponentFactory ) {
		$this->_setPrintercomponentfactory($printerComponentFactory);
		parent::__construct( $optionsArrayData, $optionsStructure);
	}
	
	/**
	 * Print options ( called walk from inherited classes )
	 */
	public function walk() {
		return $this->_walk();
	
		//return $this->_
	}
	
	protected function _oneElement( ffOneElement $item) {
		
		
		$routeForNameParam = $this->_getRouteForNameParam();
		$routeForIdParam = $this->_getRouteForIdParam();
		
		
		switch ( $item->getType() ) {
			case ffOneElement::TYPE_TABLE_START :
					$this->_getPrintercomponentfactory()->createPrinterElementTableStart()->printElement( $item, $routeForNameParam, $routeForIdParam );
				break;
				
			case ffOneElement::TYPE_TABLE_END :
					$this->_getPrintercomponentfactory()->createPrinterElementTableEnd()->printElement( $item, $routeForNameParam, $routeForIdParam );
				break;
				
			case ffOneElement::TYPE_TABLE_DATA_START :
					$this->_getPrintercomponentfactory()->createPrinterElementTableDataStart()->printElement($item, $routeForNameParam, $routeForIdParam);
				break;
				
			case ffOneElement::TYPE_TABLE_END :
					$this->_getPrintercomponentfactory()->createPrinterElementTableDataEnd()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;
					
			case ffOneElement::TYPE_NEW_LINE :
					$this->_getPrintercomponentfactory()->createPrinterElementNewLine()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;
					
			case ffOneElement::TYPE_BUTTON :
					$this->_getPrintercomponentfactory()->createPrinterElementButton()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;
					
			case ffOneElement::TYPE_HTML : 
					$this->_getPrintercomponentfactory()->createPrinterElementHtml()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;
					
			case ffOneElement::TYPE_HEADING :
					$this->_getPrintercomponentfactory()->createPrinterElementHeading()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;
					
			case ffOneElement::TYPE_PARAGRAPH:
					$this->_getPrintercomponentfactory()->createPrinterElementParagraph()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;			

			case ffOneElement::TYPE_DESCRIPTION:
					$this->_getPrintercomponentfactory()->createPrinterElementDescription()->printElement( $item, $routeForNameParam, $routeForIdParam );
					break;					
		}
	}
	
	protected function _beforeContainer( $item ) {
		
	}
	protected function _afterContainer( $item ) {
	
	}
	protected function _oneOption( $item ) {
		$routeForNameParam = $this->_getRouteForNameParam();
		$routeForIdParam = $this->_getRouteForIdParam();
		
		switch( $item->getType() ) {
			
			/***** TYPE TEXT *****/
			case ffOneOption::TYPE_TEXT :
					$this->_getPrintercomponentfactory()
							->createPrinterOptionText()
							->printOption($item, $routeForNameParam, $routeForIdParam);
				break;
				
			/***** TYPE TEXTAREA *****/
			case ffOneOption::TYPE_TEXTAREA :
					$this->_getPrintercomponentfactory()
					->createPrinterOptionTextarea()
					->printOption($item, $routeForNameParam, $routeForIdParam);
					break;				
				
			/***** TYPE NUMBER *****/
			case ffOneOption::TYPE_NUMBER :
					$this->_getPrintercomponentfactory()
							->createPrinterOptionNumber()
							->printOption($item, $routeForNameParam, $routeForIdParam);
				break;
				
			/***** TYPE SELECT *****/
			case ffOneOption::TYPE_SELECT :
					$this->_getPrintercomponentfactory()
					->createPrinterOptionSelect()
					->printOption($item, $routeForNameParam, $routeForIdParam);
				break;				
				
			/***** TYPE CHECKBOX *****/
			case ffOneOption::TYPE_CHECKBOX :
					$this->_getPrintercomponentfactory()
					->createPrinterOptionCheckbox()
					->printOption($item, $routeForNameParam, $routeForIdParam);
				break;
		}
	}
	
	protected function _getRouteForIdParam() {
		$currentRoute = implode('-', $this->_getRoute() ).'-';
		return $this->_getIdprefix() . $currentRoute;
	}
	
	protected function _getRouteForNameParam() {
		$currentRouteToArray = '[' . implode('][', $this->_getRoute()) .']';
		return $this->_getNameprefix() . $currentRouteToArray;
	}


	
	
/******************************************************************************/
/* PRIVATE FUNCTIONS
/******************************************************************************/
	
/******************************************************************************/
/* SETTERS AND GETTERS
/******************************************************************************/

	/**
	 * @return ffOptionsPrinterComponent_Factory
	 */
	protected function _getPrintercomponentfactory() {
		return $this->_printerComponentFactory;
	}
	
	/**
	 * @param ffOptionsPrinterComponent_Factory $_printerComponentFactory
	 */
	protected function _setPrintercomponentfactory(ffOptionsPrinterComponent_Factory $printerComponentFactory) {
		$this->_printerComponentFactory = $printerComponentFactory;
		return $this;
	}

	/**
	 * @return unknown_type
	 */
	protected function _getNameprefix() {
		return $this->_namePrefix;
	}
	
	/**
	 * @param unknown_type $_namePrefix
	 */
	public function setNameprefix($_namePrefix) {
		$this->_namePrefix = $_namePrefix;
		return $this;
	}
	
	/**
	 * @return unknown_type
	 */
	protected function _getIdprefix() {
		return $this->_idPrefix;
	}
	
	/**
	 * @param unknown_type $_idPrefix
	 */
	public function setIdprefix($_idPrefix) {
		$this->_idPrefix = $_idPrefix;
		return $this;
	}
	


	
}