<?php

class ffPostGetter extends ffBasicObject {
	const CUSTOM_TAXONOMY_NAME = 'taxonomy';
	const CUSTOM_TAOXNOMY_ID = 'terms';
	const CUSTOM_TAXONOMY_FIELD = 'field';
	const CUSTOM_TAXONOMY_FIELD_ID = 'id';
	
	const ARG_POSTS_PER_PAGE = 'posts_per_page';
	
	private $_postTypes = array();
	
	private $_categories = array();
	
	private $_customTaxonomies = array();
	
	/**
	 * 
	 * @var ffWPLayer
	 */
	private $_WPLayer = null;
	
	private $_currentArgs = array();
	
	
	public function __construct( ffWPLayer $WPLayer ) {
		$this->_setWPLayer($WPLayer);
	}
	
	
	public function addCategory( $categoryId ) {
		$this->_categories[] = $categoryId;
		
		return $this;
	}
	
	public function getPosts() {
		$arguments = $this->_createArguments();
		$posts = $this->_getWPLayer()->get_posts( $arguments );
		$this->_reset();
		
		return $posts;
	}
	
	private function _reset() {
		$this->_postTypes = array();
		$this->_categories = array();
		$this->_customTaxonomies = array();
		$this->_currentArgs = array();
	}
	
	public function setArgument( $name, $value ) {
		$this->_currentArgs[ $name ] = $value;
		
		return $this;
	}
	
	private function _createArguments() {
		$args = $this->_currentArgs;
		if( !empty( $this->_categories ) ) {
			$args['category'] = implode(',',$this->_categories);
		} 
		
		if( !empty( $this->_customTaxonomies ) ) {
			$args['tax_query'] = $this->_customTaxonomies;
		}
		
		if( !empty( $this->_postTypes ) ) {
			$args['post_type'] = $this->_postTypes;
		}
		
		return $args;
	}
	
	
	
	public function addCustomTaxonomy( $taxonomyName, $taxonomyId = null) {
		$customTaxonomy = array();
		
		$customTaxonomy[ ffPostGetter::CUSTOM_TAXONOMY_NAME ] = $taxonomyName;
		$customTaxonomy[ ffPostGetter::CUSTOM_TAOXNOMY_ID ] = $taxonomyId;
		$customTaxonomy[ ffPostGetter::CUSTOM_TAXONOMY_FIELD ] = ffPostGetter::CUSTOM_TAXONOMY_FIELD_ID;
		
		$this->_customTaxonomies[] = $customTaxonomy;
		
		return $this;
	}
	
	public function addPostType( $postTypeName ) {
		$this->_postTypes[] = $postTypeName;
		return $this;
	}

	/**
	 * @return ffWPLayer
	 */
	protected function _getWPLayer() {
		return $this->_WPLayer;
	}
	
	/**
	 * @param ffWPLayer $WPLayer
	 */
	protected function _setWPLayer(ffWPLayer $WPLayer) {
		$this->_WPLayer = $WPLayer;
		return $this;
	}
	
}